@@ -43,6 +43,7 @@ class AWACImpl(SACImpl):
         n_critics: int,
         bootstrap: bool,
         share_encoder: bool,
+        target_reduction_type: str,
         use_gpu: Optional[Device],
         scaler: Optional[Scaler],
         action_scaler: Optional[ActionScaler],
@@ -65,6 +66,7 @@ class AWACImpl(SACImpl):
             n_critics=n_critics,
             bootstrap=bootstrap,
             share_encoder=share_encoder,
+            target_reduction_type=target_reduction_type,
             initial_temperature=1e-20,
             use_gpu=use_gpu,
             scaler=scaler,
